@@ -197,13 +197,6 @@ bail:
 
 Bool uxa_glyphs_init(ScreenPtr pScreen)
 {
-	/* We are trying to initialise per screen resources prior to the
-	 * complete initialisation of the screen. So ensure the components
-	 * that we depend upon are initialsed prior to our use.
-	 */
-	if (!CreateScratchPixmapsForScreen(pScreen->myNum))
-		return FALSE;
-
 #if HAS_DIXREGISTERPRIVATEKEY
 	if (!dixRegisterPrivateKey(&uxa_glyph_key, PRIVATE_GLYPH, 0))
 		return FALSE;
@@ -212,6 +205,17 @@ Bool uxa_glyphs_init(ScreenPtr pScreen)
 		return FALSE;
 #endif
 
+	/* Skip pixmap creation if we don't intend to use it. */
+	if (uxa_get_screen(pScreen)->force_fallback)
+		return TRUE;
+
+	/* We are trying to initialise per screen resources prior to the
+	 * complete initialisation of the screen. So ensure the components
+	 * that we depend upon are initialsed prior to our use.
+	 */
+	if (!CreateScratchPixmapsForScreen(pScreen->myNum))
+		return FALSE;
+
 	if (!uxa_realize_glyph_caches(pScreen))
 		return FALSE;
 
@@ -293,18 +297,19 @@ uxa_glyph_cache_upload_glyph(ScreenPtr screen,
 }
 
 void
-uxa_glyph_unrealize(ScreenPtr pScreen,
-		    GlyphPtr pGlyph)
+uxa_glyph_unrealize(ScreenPtr screen,
+		    GlyphPtr glyph)
 {
 	struct uxa_glyph *priv;
 
-	priv = uxa_glyph_get_private(pGlyph);
+	/* Use Lookup in case we have not attached to this glyph. */
+	priv = dixLookupPrivate(&glyph->devPrivates, &uxa_glyph_key);
 	if (priv == NULL)
 		return;
 
 	priv->cache->glyphs[priv->pos] = NULL;
 
-	uxa_glyph_set_private(pGlyph, NULL);
+	uxa_glyph_set_private(glyph, NULL);
 	free(priv);
 }
 
@@ -780,9 +785,8 @@ uxa_glyphs_to_dst(CARD8 op,
 
 				mask_pixmap =
 					uxa_get_drawable_pixmap(this_atlas->pDrawable);
-				assert (uxa_pixmap_is_offscreen(mask_pixmap));
-
-				if (!uxa_screen->info->prepare_composite(op,
+				if (!uxa_pixmap_is_offscreen(mask_pixmap) ||
+				    !uxa_screen->info->prepare_composite(op,
 									 localSrc, this_atlas, pDst,
 									 src_pixmap, mask_pixmap, dst_pixmap))
 					return -1;
@@ -983,9 +987,8 @@ uxa_glyphs_via_mask(CARD8 op,
 
 				src_pixmap =
 					uxa_get_drawable_pixmap(this_atlas->pDrawable);
-				assert (uxa_pixmap_is_offscreen(src_pixmap));
-
-				if (!uxa_screen->info->prepare_composite(PictOpAdd,
+				if (!uxa_pixmap_is_offscreen(src_pixmap) ||
+				    !uxa_screen->info->prepare_composite(PictOpAdd,
 									 this_atlas, NULL, mask,
 									 src_pixmap, NULL, pixmap))
 					return -1;
